@@ -1,5 +1,60 @@
 #!/usr/bin/env python
 #
+
+#
+# Git Command Result on GitHub
+# ============================
+#
+# On PR to 'dev' branch
+# ---------------------
+#
+# git rev-parse --show-toplevel
+#     /home/runner/work/qiita-sync-test/qiita-sync-test
+# git config --get remote.origin.url
+#     https://github.com/ryokat3/qiita-sync-test
+# git rev-parse --abbrev-ref HEAD
+#     HEAD
+# git branch --all
+#     * (HEAD detached at pull/2/merge)
+#       remotes/pull/2/merge
+# echo ${GITHUB_REF}
+#     refs/pull/2/merge
+# git log pull/2/merge -1 --pretty=%cI README.md
+#     2022-01-10T23:08:24+00:00
+#
+#
+# On push to 'main'
+# -----------------
+#
+# git rev-parse --show-toplevel
+#     /home/runner/work/qiita-sync-test/qiita-sync-test
+# git config --get remote.origin.url
+#     https://github.com/ryokat3/qiita-sync-test
+# git rev-parse --abbrev-ref HEAD
+#     main
+# git branch --all
+#     * main
+#       remotes/origin/main
+# echo ${GITHUB_REF}
+#     refs/heads/main
+# git log heads/main -1 --pretty=%cI README.md
+#     2022-01-11T07:50:58+09:00
+#
+#
+# NOTE:
+#
+# How to get branch name (2022-01-11)
+# -----------------------------------
+#
+# When invoked on pull request, the command "git rev-parse --abbrev-ref HEAD"
+# return "HEAD", which is not a branch name (HEAD is detached).
+#
+# Instead, in GitHub Actions, environment variable "GITHUB_REF" has
+# "refs/pull/11/merge". "pull/11/merg" seems to be used as branch name.
+#
+# So, at first try "git rev-parse --abbrev-ref HEAD", and if it's 'HEAD', then get "GITHUB_REF"
+#
+
 from __future__ import annotations
 
 import functools
@@ -49,6 +104,8 @@ ACCESS_TOKEN_ENV = "QIITA_ACCESS_TOKEN"
 DEFAULT_TITLE = "No Title"
 DEFAULT_TAGS = "No Tag"
 
+GITHUB_REF = "GITHUB_REF"
+
 ########################################################################
 # Logger
 ########################################################################
@@ -203,7 +260,10 @@ def git_get_remote_url() -> str:
 
 def git_get_committer_date(filename: str) -> str:
     # "%cI", committer date, strict ISO 8601 format
-    return exec_command("git log origin/main -1 --pretty=%cI".split() + [filename])
+    head = git_get_HEAD()
+    branch = f"origin/{head}" if head != "HEAD" else head
+    result = exec_command(f"git log {branch} --pretty=%cI".split() + [filename]).splitlines()
+    return result[0]
 
 
 def git_get_committer_datetime(filename: str) -> datetime:
@@ -212,6 +272,19 @@ def git_get_committer_datetime(filename: str) -> datetime:
 
 @functools.lru_cache(maxsize=1)
 def git_get_default_branch() -> str:
+    head = git_get_HEAD()
+    if head == "HEAD":
+        github_ref = os.environ.get(GITHUB_REF)
+        if github_ref is None:
+            raise ApplicationError("Failed to get Gig branch name")
+        else:
+            return github_ref.replace('refs/', '', 1)
+    else:
+        return head
+
+
+@functools.lru_cache(maxsize=1)
+def git_get_HEAD() -> str:
     return exec_command("git rev-parse --abbrev-ref HEAD".split())
 
 
@@ -449,12 +522,18 @@ class QiitaData(NamedTuple):
 
     @classmethod
     def fromString(cls, text: str, default_title=DEFAULT_TITLE, default_tags=DEFAULT_TAGS) -> QiitaData:
-        data = dict({"title": default_title, "tags": default_tags},
-            **dict(map(lambda tpl: (tpl[0].strip(), tpl[1].strip()),
-                    map(lambda line: line.split(":", 1),
-                        filter(lambda line: re.match(r"^\s*\w+\s*:.*\S", line) is not None, text.splitlines())))))
-        return cls(data["title"], QiitaTags.fromString(
-            data["tags"]), data.get("id"), Maybe(data.get("private")).map(str2bool).getOrElse(False))
+        data = dict({
+            "title": default_title,
+            "tags": default_tags
+        },
+                    **dict(
+                        map(
+                            lambda tpl: (tpl[0].strip(), tpl[1].strip()),
+                            map(lambda line: line.split(":", 1),
+                                filter(lambda line: re.match(r"^\s*\w+\s*:.*\S", line) is not None,
+                                       text.splitlines())))))
+        return cls(data["title"], QiitaTags.fromString(data["tags"]), data.get("id"),
+                   Maybe(data.get("private")).map(str2bool).getOrElse(False))
 
     @classmethod
     def fromApi(cls, item) -> QiitaData:
@@ -488,22 +567,23 @@ class QiitaArticle(NamedTuple):
         }
 
     @classmethod
-    def fromFile(cls, filepath: Path, git_timestamp: bool = False) -> QiitaArticle:
+    def fromFile(cls, filepath: Path, git_timestamp: bool) -> QiitaArticle:
         text = filepath.read_text()
         timestamp = git_get_committer_datetime(str(filepath)) if git_timestamp else datetime.fromtimestamp(
             filepath.stat().st_mtime, timezone.utc)
         m = HEADER_REGEX.match(text)
         logger.debug(f'{filepath} :: {m.group(1) if m is not None else "None"}')
         body = Maybe(m).map(lambda m: m.group(2)).getOrElse(text)
-        data = QiitaData.fromString(Maybe(m).map(lambda m: m.group(1)).getOrElse(""),
-                qiita_get_temporary_title(body), qiita_get_temporary_tags(body))
+        data = QiitaData.fromString(
+            Maybe(m).map(lambda m: m.group(1)).getOrElse(""), qiita_get_temporary_title(body),
+            qiita_get_temporary_tags(body))
 
         return cls(data=data, body=body, timestamp=timestamp, filepath=filepath)
 
     @classmethod
     def fromApi(cls, item) -> QiitaArticle:
-        return cls(data=QiitaData.fromApi(item), body=item["body"],
-                timestamp=get_utc(item["updated_at"]), filepath=None)
+        return cls(
+            data=QiitaData.fromApi(item), body=item["body"], timestamp=get_utc(item["updated_at"]), filepath=None)
 
 
 #######################################################################
@@ -517,13 +597,16 @@ MARKDOWN_IMAGE_REGEX = re.compile(r"(\!\[[^\]]*\]\()([^\ \)]+)(.*?\))", re.MULTI
 
 
 def markdown_code_block_split(text: str) -> List[str]:
-    return list(filter(lambda elm: elm is not None and re.match(r"^````*$", elm) is None,
-                re.split(CODE_BLOCK_REGEX, text)))
+    return list(
+        filter(lambda elm: elm is not None and re.match(r"^````*$", elm) is None, re.split(CODE_BLOCK_REGEX, text)))
 
 
 def markdown_code_inline_split(text: str) -> List[str]:
-    return list(filter(None, filter(lambda elm: elm is not None and re.match(r"^``*$", elm) is None,
-                re.split(CODE_INLINE_REGEX, text))))
+    return list(
+        filter(
+            None,
+            filter(lambda elm: elm is not None and re.match(r"^``*$", elm) is None, re.split(CODE_INLINE_REGEX,
+                                                                                             text))))
 
 
 def markdown_replace_block_text(func: Callable[[str], str], text: str):
@@ -593,9 +676,8 @@ def qsync_get_local_article(include_patterns: List[str], exclude_patterns: List[
     topdir = Path(git_get_topdir())
     return [
         Path(fp).resolve()
-        for fp in (functools.reduce(
-            lambda a, b: a | b, [set(topdir.glob(pattern)) for pattern in include_patterns]) - functools.reduce(
-                lambda a, b: a | b, [set(topdir.glob(pattern)) for pattern in exclude_patterns]))
+        for fp in (functools.reduce(lambda a, b: a | b, [set(topdir.glob(pattern)) for pattern in include_patterns]) -
+                   functools.reduce(lambda a, b: a | b, [set(topdir.glob(pattern)) for pattern in exclude_patterns]))
     ]
 
 
@@ -714,8 +796,10 @@ class QiitaSync(NamedTuple):
         if article.data.id is not None:
             qiita_patch_item(self.caller, article.data.id, self.toGlobalFormat(article).toApi())
         else:
-            Maybe(qiita_post_item(self.caller, self.toGlobalFormat(article).toApi())).map(QiitaArticle.fromApi).map(
-                lambda x: article._replace(data=x.data, timestamp=x.timestamp)).map(lambda x: self.save(x))
+            Maybe(qiita_post_item(
+                self.caller,
+                self.toGlobalFormat(article).toApi())).map(QiitaArticle.fromApi).map(
+                    lambda x: article._replace(data=x.data, timestamp=x.timestamp)).map(lambda x: self.save(x))
 
     def save(self, article: QiitaArticle):
         filepath = article.filepath or Path(self.git_dir).joinpath(f"{article.data.id or 'unknown'}.md")
@@ -773,6 +857,9 @@ def qsync_show_on_diff(qsync: QiitaSync, local_article: QiitaArticle, global_art
         for diff in difflib.unified_diff(la.body.splitlines(), ga.body.splitlines()):
             print(diff)
 
+    print(f'Local Timestamp: {local_article.timestamp.strftime("%Y-%m-%dT%H:%M:%S.%f%z")}')
+    print(f'Qiita Timestamp: {global_article.timestamp.strftime("%Y-%m-%dT%H:%M:%S.%f%z")}')
+
     if local_article.timestamp > global_article.timestamp:
         if local_article.filepath is None:
             raise ApplicationError('No filepath defined')
@@ -832,7 +919,7 @@ def qsync_argparse() -> ArgumentParser:
         parser.add_argument("-i", "--include", nargs='*', default=DEFAULT_INCLUDE_GLOB, help="include glob")
         parser.add_argument("-e", "--exclude", nargs='*', default=DEFAULT_EXCLUDE_GLOB, help="exclude glob")
         parser.add_argument("-v", "--verbose", action='store_true', help="debug logging")
-        parser.add_argument("--git-timestamp", action='store_true', help="Use git time instead of mtime")
+        parser.add_argument("--file-timestamp", action='store_true', help="Use mtime instead of git time")
 
         return parser
 
@@ -852,7 +939,7 @@ def qsync_init(args) -> QiitaSync:
     access_token = qsync_get_access_token(args.token)
     local_article = qsync_get_local_article(args.include, args.exclude)
 
-    return QiitaSync.getInstance(access_token, local_article, args.git_timestamp)
+    return QiitaSync.getInstance(access_token, local_article, not args.file_timestamp)    
 
 
 def qsync_main():
